program Solve_Linear_Equation(input,output);
{======================================================}
{ Solves a linear system of equations                  }
{ Written by: Christopher L. Good                      }
{======================================================}

uses
  dos, winapi, graph, crt, strings;
const
  MAXSIZE    = 100;             { maximum number of variables }
type
  ONE_DIM = array[0..MAXSIZE] of double;
  TWO_DIM = array[0..MAXSIZE] of ^ONE_DIM;
var
  outfile          : text;
  verbose          : boolean;
  number_variables,
  i,
  j                : longint;
  total            : double;
  A_Array          : TWO_DIM;
  X_Array          : ONE_DIM;
  B_Array          : ONE_DIM;

{ --- generic linear equation solver --- }
procedure Linear_Solver(     A : TWO_DIM;
                         var X : ONE_DIM;
                             B : ONE_DIM;
                             n : longint); { size of arrays }
var
  Decomp1,
  Decomp2 : TWO_DIM;  { used to decompose arrays }

  current_x,
  current_rows,
  current_col,
  i,
  j,
  k      : longint;

begin
  {decompose system of equations and solve for X(1)-X(n) }
  for i:=1 to n do
  begin
    new(Decomp1[i]);
    new(Decomp2[i]);
    writeln('Creating Decomposition arrays[',i,']');
  end;

  for current_x := 1 to n do
  begin
    writeln('Decomposing system of equations to get X(',current_x,') ..');

    { copy A array into working array (decomp2) and B array (RHS) into last position }
    for i:=1 to n do
    begin
      for j:=1 to n do
        Decomp2[i]^[j]:=A[i]^[j];
      Decomp2[i]^[n+1]:=B[i];
    end;

    current_rows:=n;
    { go through the matrix one column at a time and eliminate each coefficient }
    for current_col:= 1 to n do
      if current_col<>current_x then
      { eliminate X(current_col) coefficient, when current_col<>current_x }
      begin

        { divide each row by its n_th coefficient (if not 0) }
        for i:=(n-current_rows+1) to n do
        begin
          if (Decomp2[i]^[current_col] <> 0.0) then
            begin
              for j:=1 to n+1 do
                Decomp1[i]^[j]:=Decomp2[i]^[j]/Decomp2[i]^[current_col];
            end
          else
            begin
              for j:=1 to n+1 do
                Decomp1[i]^[j]:=Decomp2[i]^[j];
            end;
        end;

        { print out each row as a coefficient is set to 1.0 }
        if (verbose) then
        begin
          writeln(outfile);
          writeln(outfile,'Divided each row by its #',current_col,' coefficient :');
          for i:=(n-current_rows+1) to n do
          begin
            write(outfile,'Row ',i:2,' : ');
            for j:=1 to n do
            begin
              write(outfile,Decomp1[i]^[j]:11:7,'*X(',j,') ');
              if (j<n) then write(outfile,'+');
            end;
            writeln(outfile,' = ',Decomp1[i]^[n+1]:11:7);
          end;
        end;

        { rearrange rows to get non-zero component in top row for current_col }
        if ( Decomp1[n-current_rows+1]^[current_col] = 0.0 ) then    { find a non-zero row }
        begin
          for i:= (n-current_rows+2) to n do
            if (Decomp1[i]^[current_col] <> 0.0 ) then
            begin
              { swap rows - I choose 100 as the temp swap array. }
              Decomp1[100] := Decomp1[i];
              Decomp1[i] := Decomp1[n-current_rows+1];
              Decomp1[n-current_rows+1] := Decomp1[100];

              writeln(outfile,'Swapped row ',(n-current_rows+1):2,' with row ',i:2);
            end;
        end;


        { subtract each row from the current top row to get n-1 rows ( if not 0) }
        for i:=(n-current_rows+2) to n do
        begin
          if ( Decomp1[i]^[current_col] <> 0.0) then
            begin
              for j:=1 to n+1 do
                Decomp2[i]^[j]:=Decomp1[i]^[j]-Decomp1[n-current_rows+1]^[j];
            end
          else
            begin
              for j:=1 to n+1 do
                Decomp2[i]^[j]:=Decomp1[i]^[j];
            end;
        end;
        { print out each row as a coefficient is eliminated }
        if (verbose) then
        begin
          writeln(outfile);
          writeln(outfile,'Subtracted : new row = row n - row ',n-current_rows+1,
                          ' ; X(',current_col,') now eliminated :');
          for i:=(n-current_rows+2) to n do
          begin
            write(outfile,'Row ',i:2,' : ');
            for j:=1 to n do
            begin
              write(outfile,Decomp2[i]^[j]:11:7,'*X(',j,') ');
              if (j<n) then write(outfile,'+');
            end;
            writeln(outfile,' = ',Decomp2[i]^[n+1]:11:7);
          end;
        end;

        current_rows:=current_rows-1;
      end;

    { output the final value of X(current_x) }
    X[current_x]:=Decomp2[n]^[n+1]/Decomp2[n]^[current_x];
    if (verbose) then
    begin
      writeln(outfile);
      writeln(outfile,'X(',current_x,') = ',X[current_x]:11:7);
      writeln(outfile,'---------------------------------------------------------------');
    end;
  end;
end;


begin { main }
  { create A_Array and B_Array (RHS) }
  number_variables := 5;
  for i:= 1 to number_variables do
  begin
    writeln('Creating A_array[',i,']');
    new(A_Array[i]);
  end;

  A_Array[1]^[1]:=5.0;        { coefficient of variable x(1) }
  A_Array[1]^[2]:=5.0;        { coefficient of variable x(2) }
  A_Array[1]^[3]:=0.0;        { coefficient of variable x(3) }
  A_Array[1]^[4]:=0.0;        { coefficient of variable x(4) }
  A_Array[1]^[5]:=0.0;        { coefficient of variable x(5) }
  B_Array[1]    :=5.5;        { right hand side}

  A_Array[2]^[1]:=0.0;
  A_Array[2]^[2]:=0.0;
  A_Array[2]^[3]:=1.0;
  A_Array[2]^[4]:=-1.0;
  A_Array[2]^[5]:=-1.0;
  B_Array[2]    :=0.0;

  A_Array[3]^[1]:=0.0;
  A_Array[3]^[2]:=0.0;
  A_Array[3]^[3]:=0.0;
  A_Array[3]^[4]:=2.0;
  A_Array[3]^[5]:=-1.0;
  B_Array[3]    :=0.0;

  A_Array[4]^[1]:=1.0;
  A_Array[4]^[2]:=-1.0;
  A_Array[4]^[3]:=-1.0;
  A_Array[4]^[4]:=0.0;
  A_Array[4]^[5]:=0.0;
  B_Array[4]    :=0.0;

  A_Array[5]^[1]:=0.0;
  A_Array[5]^[2]:=5.0;
  A_Array[5]^[3]:=-7.0;
  A_Array[5]^[4]:=-2.0;
  A_Array[5]^[5]:=0.0;
  B_Array[5]    :=0.0;

  { now that both arrays are set up, print this system out to file }
  assign(outfile,'linear.txt');
  rewrite(outfile);
  writeln(outfile,'System of equations to be solved :');

  for i:= 1 to number_variables do
  begin
    write(outfile,'row ',i:2,' : ');
    for j:= 1 to number_variables do
    begin
      write(outfile,A_Array[i]^[j]:11:7,'*X(',j,') ');
      if (j<>number_variables) then write(outfile,'+');
    end;
    writeln(outfile,' = ',B_Array[i]:11:7);
  end;

  { if number of variables is less than 10, print out all information to output file }
  verbose:=False;
  if (number_variables < 10) then verbose:=True;

  { call the generic linear equation solver }
  Linear_Solver(A_Array,X_Array,B_Array,number_variables);

  { print out all X's : X[1]..X[n] }
  writeln(outfile);
  for i:=1 to number_variables do
    writeln(outfile,'X(',i,') = ',X_Array[i]:11:7);
  writeln(outfile,'--------------------------------------------');

  writeln(outfile,'Check of these results :');
  for i:= 1 to number_variables do
  begin
    total:=0.0;
    for j:= 1 to number_variables do
      begin
        write(outfile,'( ',A_Array[i]^[j]:11:7,' * ',X_Array[j]:11:7,') ');
        if (j<>number_variables) then write(outfile,' + ');
        total := total + (A_Array[i]^[j] * X_Array[j]);
      end;
    writeln(outfile,' = ',total:11:7);
  end;

  close(outfile);

end.